shape = {}

shapeID = -1
goodShape = false

function shape:new(x, y, xSpeed, ySpeed)
  
  local randx = math.random(-150,150)
  local randy = math.random(-150,150)
  local r = math.random(0,255)
  local g = math.random(0,255)
  local b = math.random(0,255)


  if (xSpeed == nil) then xSpeed = randx end
  if (ySpeed == nil) then ySpeed = randy end

  if x == nil then x = 10 end
  if y == nil then y = 10 end
  
  shapeID = shapeID + 1

  local object = {
    xSpeed = xSpeed,        -- Speed on the X axis
    ySpeed = ySpeed,        -- Speed on the Y axis
    id = shapeID,           -- Unique ID of shape
    x = x,                  -- X location
    y = y,                  -- Y location
    r = r,                  -- red colour
    g = g,                  -- green colour
    b = b,                  -- blue colour
    radius = 2              -- Shape width    
  }
  setmetatable(object, { __index = shape })
  goodShape = false
  return object
end


function shape:update(dt)
  
  self.x = self.x + self.xSpeed*dt
  self.y = self.y + self.ySpeed*dt
    
  -- Check X axis for edge of map collision
  if ((self.x + (self.radius*scale)) > love.graphics.getWidth()) then
    self.xSpeed = -self.xSpeed
  elseif (self.x - (self.radius*scale) < 0) then
    self.xSpeed = -self.xSpeed
  end
  -- Check Y axis for edge of map collision
  if ((self.y + self.radius*scale) > love.graphics.getHeight()) and self.ySpeed > 0 then
    self.ySpeed = -self.ySpeed
  elseif (self.y - (self.radius*scale) < 0) and self.ySpeed < 0 then
    self.ySpeed = -self.ySpeed
  end
    
  -- Check if any shapes are colliding with each other
  -- Objects are on top of each other if the distance between them is greater than the sum of their radii.
  for di,dv in ipairs(shapes) do  -- For every shape in the shape table
    if (self.id ~= dv.id) then -- If we're not colliding with ourself
      H = math.distanceBetween(self.x, self.y, dv.x, dv.y)   -- Distance between the two circles
      local A = self.x - dv.x -- Distance of adjacent edge
      local angleOfCollision = math.deg(1/(math.cos(A/H))) -- Angle given when a right angle triangle is created with self and dv
      local sumOfRadii = (self.radius*scale) + (dv.radius*scale) -- Sum of the radii 
      local levelOfPenetration = H - sumOfRadii -- Level of penetration (-ve if collision)
      local forceOfHit = (math.max(self.xSpeed, self.ySpeed) - math.max(dv.xSpeed, dv.ySpeed))
      if levelOfPenetration < 0 and forceOfHit > 0 then
        -- super basic ejection
        if (self.x > dv.x) then
          self.x = self.x - levelOfPenetration
        else
          self.x = self.x + levelOfPenetration
        end
        if (self.y > dv.y) then
          self.y = self.y - levelOfPenetration
        else
          self.y = self.y + levelOfPenetration
        end
      
          -- Apply base speed modifications
          local temp = self.xSpeed
          if (dv.xSpeed > 0) then
            self.xSpeed = self.xSpeed - dv.xSpeed*bounciness
          else
            self.xSpeed = self.xSpeed + dv.xSpeed*bounciness
          end
          if (temp > 0) then
            dv.xSpeed = dv.xSpeed + temp*bounciness
          else
            dv.xSpeed = dv.xSpeed - temp*bounciness
          end
          temp = self.ySpeed
          if (dv.ySpeed > 0) then
            self.ySpeed = self.ySpeed - dv.ySpeed*bounciness
          else
            self.ySpeed = self.ySpeed + dv.ySpeed*bounciness
          end
          if (temp > 0) then
            dv.ySpeed = dv.ySpeed + temp*bounciness
          else
            dv.ySpeed = dv.ySpeed - temp*bounciness
          end
        --TODO
        -- Angle of rebound
        -- Ejection
      end
    end
  end
end

function shape:getEmptyX()
  return 10
end

function shape:getEmptyY() 
  return 10
end

-- Returns distance between two objects
function math.distanceBetween(x1, y1, x2, y2)
  local dy = y1 - y2
  local dx = x1 - x2
  return math.sqrt((dy)^2 + (dx)^2)
end

